/*
 * $Header$
 *
 * DESCRIPTION:
 *
 *	SQL Server Agent Jobs that are installed for Hardware Assessment  
 *	- EvaluateHardwareAssessmentCache
 *
 * Protected by US patent 6,006,034; patents pending.
 * Copyright 2011 Flexera Software LLC
 * Reproduction, adaptation, or translation without prior permission
 * is prohibited.
 */

---------------------------------------------------
--SQL SERVER JOB: EvaluateHardwareAssessmentCache
---------------------------------------------------

/*
 *
 * DESCRIPTION:
 *
 *	SQL Server Agent Job EvaluateHardwareAssessmentCache:  
 *	Used to Validate and Update Hardware Assessment Cache table
 *
 */

USE msdb
GO

DECLARE @mgsDatabaseCreateName nvarchar(256)
DECLARE @jobName nvarchar(256)
DECLARE @JobID BINARY(16)  
DECLARE @ReturnCode INT    
SELECT @ReturnCode = 0     

-- This line gets replaced in mgsDatabaseCreate with the appropriate database name.
SET @mgsDatabaseCreateName = 'ManageSoft'

SET @jobName = 'EvaluateHardwareAssessmentCache'
IF EXISTS(SELECT * FROM sysjobs	WHERE name = @JobName)
	EXEC sp_delete_job @job_name = @jobName

SET @jobName = @mgsDatabaseCreateName + ' EvaluateHardwareAssessmentCache'
IF EXISTS(SELECT * FROM sysjobs	WHERE name = @JobName)
	EXEC sp_delete_job @job_name = @jobName

IF NOT EXISTS(SELECT * FROM dbo.sysjobs WHERE [name] = @jobName)
BEGIN
	-- Create SQL server job
	PRINT 'Creating job to evaluate Hardware Assessment Cache table'
	EXEC msdb.dbo.sp_add_job 
		@job_id = @JobID OUTPUT,
		@job_name = @jobName,
		@category_name = 'Database Maintenance', 
		@description = 'Job to evaluate Hardware Assessment Cache',
		@enabled = 1,
		@notify_level_eventlog = 2 -- on failure only

	-- Attach step 1 to job
	EXEC msdb.dbo.sp_add_jobstep 
		@job_id = @JobID, 
		@step_id = 1,
	 	@step_name = 'Empty HardwareAssessmentCache',
	  	@subsystem = 'TSQL',
		@database_name = @mgsDatabaseCreateName, 
  	 	@command = 'DELETE FROM HardwareAssessmentCache',
  	 	@retry_attempts = 3,
  	 	@retry_interval = 5,
		@on_success_step_id = 2, --Step_Id=2 to go to on success
		@on_success_action = 4, --Go to step on success
		@on_fail_step_id = 2, --Step_Id=2 to go to on failure
		@on_fail_action = 4 --Go to step on failure

	-- Attach step 2 to job
	EXEC msdb.dbo.sp_add_jobstep 
		@job_id = @JobID, 
		@step_id = 2,
	 	@step_name = 'Re-evaluate HardwareAssessmentCache for all OUs under domain',
	  	@subsystem = 'TSQL',
		@database_name = @mgsDatabaseCreateName, 
  	 	@command = 'EXEC WiMiValidateNUpdateAssessmentCache ''%'', 1',
  	 	@retry_attempts = 3,
  	 	@retry_interval = 5,
		@on_success_step_id = 3, --Step_Id=3 to go to on success
		@on_success_action = 4, --Go to step on success
		@on_fail_step_id = 3, --Step_Id=3 to go to on failure
		@on_fail_action = 4 --Go to step on failure

	-- Attach step 3 to job
	EXEC msdb.dbo.sp_add_jobstep 
		@job_id = @JobID, 
		@step_id = 3,
	 	@step_name = 'Re-evaluate HardwareAssessmentCache for unknown OU',
	  	@subsystem = 'TSQL',
		@database_name = @mgsDatabaseCreateName, 
  	 	@command = 'EXEC WiMiValidateNUpdateAssessmentCache NULL, 0',
  	 	@retry_attempts = 3,
  	 	@retry_interval = 5,
		@on_success_step_id = 0, --Step_Id=none to go to on success
		@on_success_action = 1, --Quit Job on success
		@on_fail_step_id = 0, --Step_Id=none to go to on failure
		@on_fail_action = 2 --Quit Job on failure
		
	-- Attach schedule to job
	EXEC msdb.dbo.sp_add_jobschedule 
		@job_id = @JobID,  
		@name = 'Schedule Cache re-evaluation for Hardware Assessment',
		@enabled = 1,
		@freq_type = 4, -- daily
		@freq_interval = 1, --once
		@active_start_time = 000000 -- 12:00:00 AM or 100000 i.e. 1 AM

	-- Attach local server to job
	EXEC msdb.dbo.sp_add_jobserver 
		@job_id = @JobID

END
GO

---------------------------------------------------
--HardwareAssessmentRule CRUD
---------------------------------------------------

-- Use ManageSoft database
USE ManageSoft
GO

PRINT 'Creating procedures to Select, Update, Insert, Delete HardwareAssessmentRule table'
-- Select
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'WiMiHardwareAssessmentRuleSelect' AND xtype = 'P')
	DROP PROCEDURE [WiMiHardwareAssessmentRuleSelect];
GO

CREATE PROCEDURE [WiMiHardwareAssessmentRuleSelect]
AS
	SET NOCOUNT ON;
SELECT AssessmentRuleID, RuleModifiedDate, AssessmentRuleSet, AssessmentSeverityID, Class, Property, ComparisonExpr, SQLType, AggregateFn, NotKnownMessage, FailMessage, PassMessage, FriendlyName, Enabled, DefaultRule FROM HardwareAssessmentRule;
GO


-- Insert
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'WiMiHardwareAssessmentRuleInsert' AND xtype = 'P')
	DROP PROCEDURE [WiMiHardwareAssessmentRuleInsert];
GO

CREATE PROCEDURE [WiMiHardwareAssessmentRuleInsert]
(
	@RuleModifiedDate datetime,
	@AssessmentRuleSet varchar(64),
	@AssessmentSeverityID int,
	@Class varchar(256),
	@Property varchar(64),
	@ComparisonExpr varchar(1024),
	@SQLType varchar(20),
	@AggregateFn varchar(10),
	@NotKnownMessage nvarchar(256),
	@FailMessage nvarchar(256),
	@PassMessage nvarchar(256),
	@FriendlyName nvarchar(256),
	@Enabled bit,
	@DefaultRule bit
)
AS
	SET NOCOUNT OFF;
INSERT INTO HardwareAssessmentRule(RuleModifiedDate, AssessmentRuleSet, AssessmentSeverityID, Class, Property, ComparisonExpr, SQLType, AggregateFn, NotKnownMessage, FailMessage, PassMessage, FriendlyName, Enabled, DefaultRule) VALUES (@RuleModifiedDate, @AssessmentRuleSet, @AssessmentSeverityID, @Class, @Property, @ComparisonExpr, @SQLType, @AggregateFn, @NotKnownMessage, @FailMessage, @PassMessage, @FriendlyName, @Enabled, @DefaultRule);
	SELECT AssessmentRuleID, RuleModifiedDate, AssessmentRuleSet, AssessmentSeverityID, Class, Property, ComparisonExpr, SQLType, AggregateFn, NotKnownMessage, FailMessage, PassMessage, FriendlyName, Enabled, DefaultRule FROM HardwareAssessmentRule WHERE (AssessmentRuleID = @@IDENTITY);
GO


-- Update
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'WiMiHardwareAssessmentRuleUpdate' AND xtype = 'P')
	DROP PROCEDURE [WiMiHardwareAssessmentRuleUpdate];
GO

CREATE PROCEDURE [WiMiHardwareAssessmentRuleUpdate]
(
	@RuleModifiedDate datetime,
	@AssessmentRuleSet varchar(64),
	@AssessmentSeverityID int,
	@Class varchar(256),
	@Property varchar(64),
	@ComparisonExpr varchar(1024),
	@SQLType varchar(20),
	@AggregateFn varchar(10),
	@NotKnownMessage nvarchar(256),
	@FailMessage nvarchar(256),
	@PassMessage nvarchar(256),
	@FriendlyName nvarchar(256),
	@Enabled bit,
	@DefaultRule bit,
	@Original_AssessmentRuleID int,
	@Original_AggregateFn varchar(10),
	@Original_AssessmentRuleSet varchar(64),
	@Original_AssessmentSeverityID int,
	@Original_Class varchar(256),
	@Original_ComparisonExpr varchar(1024),
	@Original_DefaultRule bit,
	@Original_Enabled bit,
	@Original_FailMessage nvarchar(256),
	@Original_FriendlyName nvarchar(256),
	@Original_NotKnownMessage nvarchar(256),
	@Original_PassMessage nvarchar(256),
	@Original_Property varchar(64),
	@Original_RuleModifiedDate datetime,
	@Original_SQLType varchar(20),
	@AssessmentRuleID int
)
AS
	SET NOCOUNT OFF;
	UPDATE HardwareAssessmentRule SET RuleModifiedDate = @RuleModifiedDate, AssessmentRuleSet = @AssessmentRuleSet, AssessmentSeverityID = @AssessmentSeverityID, Class = @Class, Property = @Property, ComparisonExpr = @ComparisonExpr, SQLType = @SQLType, AggregateFn = @AggregateFn, NotKnownMessage = @NotKnownMessage, FailMessage = @FailMessage, PassMessage = @PassMessage, FriendlyName = @FriendlyName, Enabled = @Enabled, DefaultRule = @DefaultRule WHERE (AssessmentRuleID = @Original_AssessmentRuleID) AND (AggregateFn = @Original_AggregateFn OR @Original_AggregateFn IS NULL AND AggregateFn IS NULL) AND (AssessmentRuleSet = @Original_AssessmentRuleSet OR @Original_AssessmentRuleSet IS NULL AND AssessmentRuleSet IS NULL) AND (AssessmentSeverityID = @Original_AssessmentSeverityID) AND (Class = @Original_Class) AND (ComparisonExpr = @Original_ComparisonExpr) AND (DefaultRule = @Original_DefaultRule) AND (Enabled = @Original_Enabled) AND (FailMessage = @Original_FailMessage) AND (FriendlyName = @Original_FriendlyName OR @Original_FriendlyName IS NULL AND FriendlyName IS NULL) AND (NotKnownMessage = @Original_NotKnownMessage) AND (PassMessage = @Original_PassMessage) AND (Property = @Original_Property) AND (RuleModifiedDate = @Original_RuleModifiedDate) AND (SQLType = @Original_SQLType OR @Original_SQLType IS NULL AND SQLType IS NULL);
	SELECT AssessmentRuleID, RuleModifiedDate, AssessmentRuleSet, AssessmentSeverityID, Class, Property, ComparisonExpr, SQLType, AggregateFn, NotKnownMessage, FailMessage, PassMessage, FriendlyName, Enabled, DefaultRule FROM HardwareAssessmentRule WHERE (AssessmentRuleID = @AssessmentRuleID);
GO


-- Delete
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'WiMiHardwareAssessmentRuleDelete' AND xtype = 'P')
	DROP PROCEDURE [WiMiHardwareAssessmentRuleDelete];
GO

CREATE PROCEDURE [WiMiHardwareAssessmentRuleDelete]
(
	@Original_AssessmentRuleID int,
	@Original_AggregateFn varchar(10),
	@Original_AssessmentRuleSet varchar(64),
	@Original_AssessmentSeverityID int,
	@Original_Class varchar(256),
	@Original_ComparisonExpr varchar(1024),
	@Original_DefaultRule bit,
	@Original_Enabled bit,
	@Original_FailMessage nvarchar(256),
	@Original_FriendlyName nvarchar(256),
	@Original_NotKnownMessage nvarchar(256),
	@Original_PassMessage nvarchar(256),
	@Original_Property varchar(64),
	@Original_RuleModifiedDate datetime,
	@Original_SQLType varchar(20)
)
AS
	SET NOCOUNT OFF;
	-- delete the data in the cache for this rule first
	DELETE FROM HardwareAssessmentCache WHERE AssessmentRuleID = @Original_AssessmentRuleID;
	DELETE FROM HardwareAssessmentRule WHERE (AssessmentRuleID = @Original_AssessmentRuleID) AND (AggregateFn = @Original_AggregateFn OR @Original_AggregateFn IS NULL AND AggregateFn IS NULL) AND (AssessmentRuleSet = @Original_AssessmentRuleSet OR @Original_AssessmentRuleSet IS NULL AND AssessmentRuleSet IS NULL) AND (AssessmentSeverityID = @Original_AssessmentSeverityID) AND (Class = @Original_Class) AND (ComparisonExpr = @Original_ComparisonExpr) AND (DefaultRule = @Original_DefaultRule) AND (Enabled = @Original_Enabled) AND (FailMessage = @Original_FailMessage) AND (FriendlyName = @Original_FriendlyName OR @Original_FriendlyName IS NULL AND FriendlyName IS NULL) AND (NotKnownMessage = @Original_NotKnownMessage) AND (PassMessage = @Original_PassMessage) AND (Property = @Original_Property) AND (RuleModifiedDate = @Original_RuleModifiedDate) AND (SQLType = @Original_SQLType OR @Original_SQLType IS NULL AND SQLType IS NULL);
GO

